import { Inject } from '@nestjs/common';
import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { DeleteAuthenticationCommand } from './delete-authentication.command';
import { AUTHENTICATION_REPOSITORY } from '@modules/authentication/authentication.di-tokens';
import { AuthenticationRepositoryPort } from '../../ports/authentication.repository.port';
import { AuthenticationEntity } from '@modules/authentication/core/domain/authentication.entity';

@CommandHandler(DeleteAuthenticationCommand)
export class DeleteAuthenticationService implements ICommandHandler {
  constructor(
    @Inject(AUTHENTICATION_REPOSITORY)
    private readonly authenticationRepository: AuthenticationRepositoryPort,
  ) {}

  async execute(command: DeleteAuthenticationCommand): Promise<boolean> {
    const authentication: AuthenticationEntity =
      await this.authenticationRepository.findOneById(command.userId, {
        usernames: true,
      });
    authentication.delete();
    const isDeleted: boolean =
      await this.authenticationRepository.delete(authentication);
    return isDeleted;
  }
}
