import {
  AggregateID,
  IdResponse,
  RpcExceptionCode,
  RpcValidationPipe,
} from '@mobicoop/ddd-library';
import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { UpdatePasswordRequestDto } from './dtos/update-password.request.dto';
import { UpdatePasswordCommand } from '@modules/authentication/core/application/commands/update-password/update-password.command';
import { GRPC_AUTHENTICATION_SERVICE_NAME } from '@src/app.constants';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class UpdatePasswordGrpcController {
  constructor(private readonly commandBus: CommandBus) {}

  @GrpcMethod(GRPC_AUTHENTICATION_SERVICE_NAME, 'UpdatePassword')
  async updatePassword(data: UpdatePasswordRequestDto): Promise<IdResponse> {
    try {
      const aggregateID: AggregateID = await this.commandBus.execute(
        new UpdatePasswordCommand({
          userId: data.userId,
          password: data.password,
        }),
      );
      return new IdResponse(aggregateID);
    } catch (error: any) {
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: error.message,
      });
    }
  }
}
