import { PasswordEncrypterPort } from '@modules/authentication/core/application/ports/password-encrypter.port';
import { EncryptionAlgorithm } from '@modules/authentication/core/domain/username.types';
import { PasswordEncrypter } from '@modules/authentication/infrastructure/password-encrypter';

describe('Password encrypter', () => {
  it('should encrypt a password in bcrypt', async () => {
    const passwordEncrypter: PasswordEncrypterPort = new PasswordEncrypter(
      EncryptionAlgorithm.BCRYPT,
    );
    const encryptedPassword: string =
      await passwordEncrypter.encrypt('somePassword');
    expect(encryptedPassword.substring(0, 2)).toBe('$2');
  });

  it('should encrypt a password in argon2i', async () => {
    const passwordEncrypter: PasswordEncrypterPort = new PasswordEncrypter(
      EncryptionAlgorithm.ARGON2I,
    );
    const encryptedPassword: string =
      await passwordEncrypter.encrypt('somePassword');
    expect(encryptedPassword.substring(0, 9)).toBe('$argon2i$');
  });

  it('should encrypt a password in argon2d', async () => {
    const passwordEncrypter: PasswordEncrypterPort = new PasswordEncrypter(
      EncryptionAlgorithm.ARGON2D,
    );
    const encryptedPassword: string =
      await passwordEncrypter.encrypt('somePassword');
    expect(encryptedPassword.substring(0, 9)).toBe('$argon2d$');
  });

  it('should encrypt a password in argon2id', async () => {
    const passwordEncrypter: PasswordEncrypterPort = new PasswordEncrypter(
      EncryptionAlgorithm.ARGON2ID,
    );
    const encryptedPassword: string =
      await passwordEncrypter.encrypt('somePassword');
    expect(encryptedPassword.substring(0, 10)).toBe('$argon2id$');
  });
});
