import { PasswordVerifierPort } from '@modules/authentication/core/application/ports/password-verifier.port';
import { PasswordVerifier } from '@modules/authentication/infrastructure/password-verifier';

describe('Password verifier', () => {
  const passwordVerifier: PasswordVerifierPort = new PasswordVerifier();

  it('should verify a bcrypt password', async () => {
    const bcryptEncryptedPassword: string =
      '$2b$10$IGMj7/tH66kzO8rqwdogK.tgY2nFOdtMC.dzMAYUVfbSTPwWk7sk.';
    const verified: boolean = await passwordVerifier.verify(
      'somePassword',
      bcryptEncryptedPassword,
    );
    expect(verified).toBeTruthy();
  });

  it('should verify an argon2i password', async () => {
    const argon2iEncryptedPassword: string =
      '$argon2i$v=19$m=16,t=2,p=1$c2lERkU0UmpMYkhSa1h3eQ$hSyr7TZZDzvq9XJgA+D/pw';
    const verified: boolean = await passwordVerifier.verify(
      'somePassword',
      argon2iEncryptedPassword,
    );
    expect(verified).toBeTruthy();
  });

  it('should verify an argon2d password', async () => {
    const argon2dEncryptedPassword: string =
      '$argon2d$v=19$m=16,t=2,p=1$c2lERkU0UmpMYkhSa1h3eQ$YBopyKamOUl7ZPhu+KPASw';
    const verified: boolean = await passwordVerifier.verify(
      'somePassword',
      argon2dEncryptedPassword,
    );
    expect(verified).toBeTruthy();
  });

  it('should verify an argon2id password', async () => {
    const argon2idEncryptedPassword: string =
      '$argon2id$v=19$m=16,t=2,p=1$c2lERkU0UmpMYkhSa1h3eQ$Heokt0Lk6mtBmRE07q94lw';
    const verified: boolean = await passwordVerifier.verify(
      'somePassword',
      argon2idEncryptedPassword,
    );
    expect(verified).toBeTruthy();
  });
});
